﻿/* ***********************************************************************
This class was created for an ActionScript 3 Tutorial
by Barbara Kaskosz at:

www.flashandmath.com

The ParamArtBoard class is a modification of the GraphingBoard class
by Barbara Kaskosz and Doug Ensley in the package bkde.as3.boards.
The GraphingBoard and the other classses in the package were originally
published at the MathDL Flash Forum in the article: 
'Flash Tools for Developers: Graphing Functions and Curves on the Plane'
by Barbara Kaskosz and Doug Ensley .

Last modified: May 1, 2008
************************************************************************ */

package bkde.as3.boards {
	
    import flash.display.*;
  
    import flash.events.*;
   
    import flash.text.*;
	
	import bkde.as3.parsers.*;

  public class ParamArtBoard extends Sprite {
	
	protected var nXSize:Number;
	
	protected var nYSize:Number;
	
	protected var nMaxNumGraphs:int;
	
	protected var spGraphsHolder:Sprite;
	
	protected var aGraphs:Array;
	
	protected var spBoard:Sprite;
	
	protected var shBorder:Shape;
	
	protected var spMask:Sprite;
	
	protected var nBackColor:Number;
	
	protected var nBorderColor:Number;
	
	protected var nBorderThick:Number;
	
	protected var nXmin:Number;
	
	protected var nXmax:Number;
	
	protected var nYmin:Number;
	
	protected var nYmax:Number;
	
	protected var procFun:MathParser;
	
	protected var procRange:RangeParser;
	
	protected var compObj1:CompiledObject;
	
    protected var compObj2:CompiledObject;
	
    protected var nTmin:Number;
	
    protected var nTmax:Number;

    protected var nAmin:Number;
	
    protected var nAmax:Number;

    protected var nBmin:Number;
	
    protected var nBmax:Number;
	
	public var ErrorBox:TextField;
	
	protected var bIsReady:Boolean;
	
	public var numPoints:Number;
		
	
	public function ParamArtBoard(w:Number,h:Number){
		
		this.nXSize=w;
		
		this.nYSize=h;
	
		this.nBackColor=0xFFFFFF;
		
		this.nBorderColor=0x000000;
		
		this.nBorderThick=1;
		
		this.procFun=new MathParser(["t","a","b"]);
		
		this.bIsReady=false;
		
		this.procRange=new RangeParser();

        this.numPoints=200;
	
		spBoard=new Sprite();
		
		this.addChild(spBoard);
		
		spMask=new Sprite();
		
		this.addChild(spMask);
		
		shBorder=new Shape();
		
		this.addChild(shBorder);
		
		drawBoard();
		
		drawMask();
		
		spBoard.mask=spMask;
		
		spGraphsHolder=new Sprite();
		
		spBoard.addChild(spGraphsHolder);
		
		setMaxNumGraphs(10);
		
		ErrorBox=new TextField();
		
		this.addChild(ErrorBox);
		
		setUpErrorBox();
		
		setErrorBoxFormat(0xFFFFFF,0xFFFFFF,0x000000,12);
		
		setErrorBoxSizeAndPos(nXSize-20,nYSize/3,10,10);
		
		
	}
	
	
	public function get isReady():Boolean {
		
		return bIsReady;
		
	}
	
	public function get xmin():Number {
		
		return nXmin;
		
	}
	
	public function get xmax():Number {
		
		return nXmax;
		
	}
	
	public function get ymin():Number {
		
		return nYmin;
		
	}
	
	public function get ymax():Number {
		
		return nYmax;
		
	}
	
	public function get tmin():Number {
		
		return nTmin;
		
	}
	
	public function get tmax():Number {
		
		return nTmax;
		
	}
	
	public function get amin():Number {
		
		return nAmin;
		
	}
	
	public function get amax():Number {
		
		return nAmax;
		
	}
	
	public function get bmin():Number {
		
		return nBmin;
		
	}
	
	public function get bmax():Number {
		
		return nBmax;
		
	}
	
	
	protected function callError(mes:String):void {
		
		ErrorBox.visible=true;
		
		ErrorBox.text=mes;
		
	}
	
	
	public function procInput(f1,f2,x1,x2,y1,y2,t1,t2,a1,a2,b1,b2):void {
	
	    var oRange:RangeObject;
	
	    ErrorBox.visible=false;
		
		bIsReady=false;
	
	   if(f1.length==0){
		
		  callError("Give formulas for both coordinates.");
		
		  return;
		
	    }
	
	   if(f2.length==0){
	
		  callError("Give formulas for both coordinates.");
		
		  return;
		
	   }
	
	   oRange=procRange.parseRangeTwo(t1,t2);
	
       if(oRange.errorStatus==1){
		
		  callError("Error in t-range.");
		
		  return;
	  }
	  
	   setParamRange(oRange.Values[0],oRange.Values[1]);
	
	   oRange=procRange.parseRangeFour(x1,x2,y1,y2);
	
       if(oRange.errorStatus==1){
		
		 callError("Error in xy-ranges.");
		
		 return;
		
	  }
	
	   setVarsRanges(oRange.Values[0],oRange.Values[1],oRange.Values[2],oRange.Values[3]);
		
       oRange=procRange.parseRangeFour(a1,a2,b1,b2);
	
        if(oRange.errorStatus==1){
		
		  callError("Error in ab-ranges.");
		
		  return;
		
		
	}
	
	  setConstsRanges(oRange.Values[0],oRange.Values[1],oRange.Values[2],oRange.Values[3]);
		
	
	  compObj1=procFun.doCompile(f1);
	  
	  if(compObj1.errorStatus==1){
		  
		  callError("Error in the formula for the first coordinate. "+compObj1.errorMes);
		
		  return;
		
	}
	
	
	compObj2=procFun.doCompile(f2);
	
	if(compObj2.errorStatus==1){
				
		 callError("Error in the formula for the second coordinate. "+compObj2.errorMes);
		
		 return;
			
	}
	
	bIsReady=true;
	
}
	
  protected function prepGraph(p:Number,d:Number):Array {
	  
	   var fArray:Array=[];
	  
	   if(!bIsReady){
		   
		  return fArray; 
	  }
	  
	  var curx:Number;
	
	  var cury:Number;

	  var i:int;
	
	  var tstep:Number;
	
	  var colo:Number=chooseColor();
	
	  tstep=(nTmax-nTmin)/numPoints;
	
	for(i=0;i<=numPoints;i++){
		
		curx=procFun.doEval(compObj1.PolishArray,[nTmin+tstep*i,p,d]);
		
		cury=procFun.doEval(compObj2.PolishArray,[nTmin+tstep*i,p,d]);
		
	    fArray[i]=[curx,cury];
		
	}
	  
	  
	  return fArray;
  }
  
  
   protected function chooseColor():Number {
	  
	   var r1:Number=Math.floor(Math.random()*256);
	
	   var r2:Number=Math.floor(Math.random()*256);
	
	   var r3:Number=Math.floor(Math.random()*256);
	
	  return combineRGB(r1,r2,r3);
  }
  
  protected function combineRGB(red:Number,green:Number,blue:Number):Number {
	
	  var RGB:Number;
	
	  if(red>255){red=255;}
	  if(green>255){green=255;}
	  if(blue>255){blue=255;}
	
	  if(red<0){red=0;}
	  if(green<0){green=0;}
	  if(blue<0){blue=0;}
	
	
	  RGB=(red<<16) | (green<<8) | blue;
	
	 return RGB;	
	
  }

	
	
  public function setMaxNumGraphs(a:int):void {
	  
	  var i:int;
	  
	  aGraphs=[];
	  
	  nMaxNumGraphs=a;
	  
	  for(i=0;i<nMaxNumGraphs;i++){
		  
		 aGraphs[i]=new Shape();
		 
		 spGraphsHolder.addChild(aGraphs[i]);
		  
	  }
	  
	   
   }
   
   
    public function getMaxNumGraphs():int {
	  
	   return nMaxNumGraphs;
	  	   
   }
   
   public function drawGraph(num:int,thick:Number,a:Number,b:Number):void {
		
		 if(!bIsReady){
		   
		  return;
		  
	     }
		
		var i:int;
		
		var colo:Number=chooseColor();
		
		var aVals:Array=prepGraph(a,b);
		
		var valLen:Number=aVals.length;
		
		var pixVals:Array=[];
		
		
		if(num<1 || num>nMaxNumGraphs){
			
			return;
			
		}
		
		for(i=0;i<valLen;i++){
			
			pixVals[i]=[xtoPix(aVals[i][0]),ytoPix(aVals[i][1])];
			
		}
			
		aGraphs[num-1].graphics.clear();
			
		aGraphs[num-1].graphics.lineStyle(thick,colo);
		
		for(i=0;i<valLen-1;i++){
			
			if(isDrawable(pixVals[i][1]) && isDrawable(pixVals[i+1][1]) && isDrawable(pixVals[i][0]) && isDrawable(pixVals[i+1][0])){
		
		aGraphs[num-1].graphics.moveTo(pixVals[i][0],pixVals[i][1]);
		
	    aGraphs[num-1].graphics.lineTo(pixVals[i+1][0],pixVals[i+1][1]);
		
		   }
		
		}
		
		
	}
	
	
	public function eraseGraphs():void {
		
		var i:int;
		
		for(i=0; i<nMaxNumGraphs; i++){
				
			aGraphs[i].graphics.clear();
			
		}
			
	}
	
	
	
	public function resetBoard():void {
		
		 eraseGraphs();
		
		 ErrorBox.visible=false;
		 
		 bIsReady=false;
		
	}
	
	
	
	
  protected function drawMask():void {
	
	spMask.graphics.lineStyle(1,0x000000);
	
	spMask.graphics.beginFill(0xFFFFFF);
	
	spMask.graphics.moveTo(0,0);
	
	spMask.graphics.lineTo(nXSize+1,1);
	
	spMask.graphics.lineTo(nXSize+1,nYSize+1);
	
	spMask.graphics.lineTo(0,nYSize+1);
	
	spMask.graphics.lineTo(0,0);
	
	spMask.graphics.endFill();
		
}



  protected function drawBoard():void {
	  
	spBoard.graphics.clear();
	
	spBoard.graphics.beginFill(nBackColor);
	
	spBoard.graphics.moveTo(0,0);
	
	spBoard.graphics.lineTo(nXSize,0);
	
	spBoard.graphics.lineTo(nXSize,nYSize);
	
	spBoard.graphics.lineTo(0,nYSize);
	
	spBoard.graphics.lineTo(0,0);
	
	spBoard.graphics.endFill();
	
	drawBorder();
	
		
}

   protected function drawBorder():void {
	   
	   shBorder.graphics.clear();
	   
	   shBorder.graphics.lineStyle(nBorderThick,nBorderColor);
	   
	   shBorder.graphics.moveTo(0,0);
	
	   shBorder.graphics.lineTo(nXSize,0);
	
	   shBorder.graphics.lineTo(nXSize,nYSize);
	
	   shBorder.graphics.lineTo(0,nYSize);
	
	   shBorder.graphics.lineTo(0,0);
	   
   }

  
	public function changeBorderColorAndThick(colo:Number,t:Number): void {
		
		nBorderColor=colo;
		
		nBorderThick=t;
		
		drawBoard();
		
	}
	
	public function changeBackColor(colo:Number): void {
		
		nBackColor=colo;
		
		drawBoard();
		
	}
	
	
	public function getBoardWidth():Number {
		
		return nXSize;
		
	}
	
	public function getBoardHeight():Number {
		
		return nYSize;
		
	}
	
	
	protected function setVarsRanges(a:Number,b:Number,c:Number,d:Number): void {
		
		          nXmin=a;
		
		          nXmax=b;
		
		          nYmin=c;
		
		          nYmax=d;
		
	}
	
	protected function setConstsRanges(a:Number,b:Number,c:Number,d:Number): void {
		
		          nAmin=a;
		
		          nAmax=b;
		
		          nBmin=c;
		
		          nBmax=d;
		
	}
	
	protected function setParamRange(a:Number,b:Number): void {
		
		          nTmin=a;
		
		          nTmax=b;
		
		       
	}
	
	public function isDrawable(a:*):Boolean {
		
		
		if((typeof a)!="number" || isNaN(a) || !isFinite(a)){
			
			return false; } 
			
			
		if(Math.abs(a)>=5000){
			
			return false;
		}
		
		
		return true;
		
	}
	
	
	public function xtoPix(a:Number): Number {
		
		var xconv:Number;
		
		if(bIsReady){
		
		xconv=nXSize/(nXmax-nXmin);
		
		return (a-nXmin)*xconv;
		
		} else {
			
			return NaN;
			
		}
		
		
	}
	
	public function ytoPix(a:Number): Number {
		
		var yconv:Number;
		
		if(bIsReady){
		
		  yconv=nYSize/(nYmax-nYmin);
		
		return nYmax*yconv-a*yconv;
		
		} else {
			
			return NaN;
			
		}
		
	}
	
	public function xtoFun(a:Number): Number {
		
		var xconv:Number;
		
		if(bIsReady){
		
		   xconv=nXSize/(nXmax-nXmin);
		
		return a/xconv+nXmin;
		
		} else {
			
			return NaN;
			
		}
		
		
		
	}
	
	public function ytoFun(a:Number): Number {
		
		var yconv:Number;
		
		if(bIsReady){
		
		   yconv=nYSize/(nYmax-nYmin);
		
		return nYmax-a/yconv;	
		
		} else {
			
			return NaN;
			
		}
		
	}
	
	protected function setUpErrorBox():void {
		
		ErrorBox.type=TextFieldType.DYNAMIC;
		
		ErrorBox.wordWrap=true;
		
		ErrorBox.border=true;
		
		ErrorBox.background=true;
		
		ErrorBox.text="";
		
		ErrorBox.visible=false;
		
		ErrorBox.mouseEnabled=false;
		
		
	}
	
	public function setErrorBoxSizeAndPos(w:Number,h:Number,a:Number,b:Number): void {
		
				
		ErrorBox.width=w;
		
		ErrorBox.height=h;
		
		ErrorBox.x=a;
		
		ErrorBox.y=b;
		
	}
	
	
	
	public function setErrorBoxFormat(colo1:Number,colo2:Number,colo3:Number,s:Number): void {
		
		var errorFormat:TextFormat=new TextFormat();

        errorFormat.color=colo3;

        errorFormat.size=s;

        errorFormat.font="Arial";

        ErrorBox.defaultTextFormat=errorFormat;
		
		ErrorBox.backgroundColor=colo1;
			
		ErrorBox.borderColor=colo2;
		
	}
	

	public function destroy():void {
		
		var i:int;
		
		var countBoard:int=spBoard.numChildren;
		
		var countHolder:int=spGraphsHolder.numChildren;
		
		var countMain:int=this.numChildren;
			
		spBoard.mask=null;
		
		shBorder.graphics.clear();
		
		spBoard.graphics.clear();
		
		spMask.graphics.clear();
		
		for(i=0;i<nMaxNumGraphs;i++){
			
			aGraphs[i].graphics.clear();
			
		}
		
		for(i=0;i<countHolder;i++){
			
			spGraphsHolder.removeChildAt(0);
			
		}
		
		for(i=0;i<countBoard;i++){
			
			spBoard.removeChildAt(0);
			
		}
		
		for(i=0;i<countMain;i++){
			
			this.removeChildAt(0);
			
		}
		
		spMask=null;
		
		spBoard=null;
		
		spGraphsHolder=null;
		
		ErrorBox=null;
		
		spMask=null;
		
		shBorder=null;
		
	}
		
	
}

}

